@@ -684,7 +684,7 @@ loop(void)
 		FD_SET(tty_fd, &rdset);
 		if ( tty_q.len ) FD_SET(tty_fd, &wrset);
 
-		if (select(FD_SETSIZE, &rdset, &wrset, NULL, NULL) < 0)
+		if (select(tty_fd + 1, &rdset, &wrset, NULL, NULL) < 0)
 			fatal("select failed: %d : %s", errno, strerror(errno));
 
 		if ( FD_ISSET(STI, &rdset) ) {
@@ -694,10 +694,15 @@ loop(void)
 			do {
 				n = read(STI, &c, 1);
 			} while (n < 0 && errno == EINTR);
-			if (n == 0)
+			if (n == 0) {
 				fatal("stdin closed");
-			else if (n < 0)
-				fatal("read from stdin failed: %s", strerror(errno));
+			} else if (n < 0) {
+				/* is this really necessary? better safe than sory! */
+				if ( errno != EAGAIN && errno != EWOULDBLOCK ) 
+					fatal("read from stdin failed: %s", strerror(errno));
+				else
+					goto skip_proc_STI;
+			}
 
 			switch (state) {
 
@@ -812,7 +817,7 @@ loop(void)
 					if ( r <= -1 )
 						fatal("cannot read filename: %s", strerror(errno));
 					if ( fname[0] )
-						run_cmd(tty_fd, opts.send_cmd, fname, NULL);
+						run_cmd(tty_fd, opts.receive_cmd, fname, NULL);
 					else
 						run_cmd(tty_fd, opts.receive_cmd, NULL);
 					break;
@@ -845,6 +850,7 @@ loop(void)
 				break;
 			}
 		}
+	skip_proc_STI:
 
 		if ( FD_ISSET(tty_fd, &rdset) ) {
 
@@ -853,12 +859,14 @@ loop(void)
 			do {
 				n = read(tty_fd, &c, 1);
 			} while (n < 0 && errno == EINTR);
-			if (n == 0)
+			if (n == 0) {
 				fatal("term closed");
-			else if ( n < 0 )
-				fatal("read from term failed: %s", strerror(errno));
-			
-			map_and_write(STO, opts.imap, c);
+			} else if ( n < 0 ) {
+				if ( errno != EAGAIN && errno != EWOULDBLOCK )
+					fatal("read from term failed: %s", strerror(errno));
+			} else {
+				map_and_write(STO, opts.imap, c);
+			}
 		}
 
 		if ( FD_ISSET(tty_fd, &wrset) ) {
@@ -1028,7 +1036,9 @@ parse_args(int argc, char *argv[])
 			opts.noreset = 1;
 			break;
 		case 'l':
+#ifdef UUCP_LOCK_DIR
 			opts.nolock = 1;
+#endif
 			break;
 		case 'e':
 			opts.escape = optarg[0] & 0x1f;
@@ -1128,7 +1138,9 @@ parse_args(int argc, char *argv[])
 	printf("local echo is  : %s\n", opts.lecho ? "yes" : "no");
 	printf("noinit is      : %s\n", opts.noinit ? "yes" : "no");
 	printf("noreset is     : %s\n", opts.noreset ? "yes" : "no");
+#ifdef UUCP_LOCK_DIR
 	printf("nolock is      : %s\n", opts.nolock ? "yes" : "no");
+#endif
 	printf("send_cmd is    : %s\n", opts.send_cmd);
 	printf("receive_cmd is : %s\n", opts.receive_cmd);
 	printf("imap is        : "); print_map(opts.imap);
@@ -1159,7 +1171,7 @@ main(int argc, char *argv[])
 		fatal("cannot lock %s: %s", opts.port, strerror(errno));
 #endif
 
-	tty_fd = open(opts.port, O_RDWR | O_NONBLOCK);
+	tty_fd = open(opts.port, O_RDWR | O_NONBLOCK | O_NOCTTY);
 	if (tty_fd < 0)
 		fatal("cannot open %s: %s", opts.port, strerror(errno));
 
