@@ -627,9 +627,17 @@ term_set_baudrate (int fd, int baudrate)
 		case 115200:
 			spd = B115200;
 			break;
+#ifdef HIGH_BAUD
 		case 230400:
 			spd = B230400;
 			break;
+		case 460800:
+			spd = B460800;
+			break;
+		case 921600:
+			spd = B921600;
+			break;
+#endif
 		default:
 			term_errno = TERM_EBAUD;
 			rval = -1;
@@ -661,7 +669,7 @@ term_set_baudrate (int fd, int baudrate)
 /***************************************************************************/
 
 int
-term_set_parity (int fd, enum parity_e parity)
+term_set_parity (int fd, enum parity_e parity) 
 {
 	int rval, i;
 	struct termios *tiop;
@@ -680,13 +688,14 @@ term_set_parity (int fd, enum parity_e parity)
 
 		switch (parity) {
 		case P_EVEN:
-			tiop->c_cflag = (tiop->c_cflag & ~(PARENB | PARODD)) | PARENB;
+			tiop->c_cflag &= ~PARODD;
+			tiop->c_cflag |= PARENB;
 			break;
 		case P_ODD:
-			tiop->c_cflag = (tiop->c_cflag & ~(PARENB | PARODD)) | PARODD;
+			tiop->c_cflag |= PARENB | PARODD;
 			break;
 		case P_NONE:
-			tiop->c_cflag = (tiop->c_cflag & ~(PARENB | PARODD));
+			tiop->c_cflag &= ~(PARENB | PARODD); 
 			break;
 		default:
 			term_errno = TERM_EPARITY;
